"""
The helper module

Here you'll find some useful functions and classes for interacting with fotoobo. Mainly there are
configuration, logging and input/output functions.
Put everything into this module what does not belong directly to any other module.
"""

from typing import List

cli_path: List[str] = []
